# Literals for boto3 {{ service_name.class_name }} module

> [Index](..) > [{{ service_name.class_name }}](.) > Literals

Auto-generated documentation for [{{ service_name.class_name }}]({{ service_name.boto3_doc_link }})
type annotations stubs module [{{ service_name.module_name }}]({{ service_name.pypi_link }}).

{% for literal in package.literals -%}
## {{ literal.name }}

```python
from {{ service_name.module_name }}.literals import {{ literal.name }}
```

Values:

{% for child in literal.children|sort -%}
- `{{ child }}`{{ '\n' -}}
{% endfor -%}
{{ '\n' -}}
{% endfor %}