# Type annotations for boto3

Auto-generated documentation for [boto3](https://boto3.amazonaws.com/v1/documentation/api/latest/index.html) type annotations package [boto3-stubs](https://pypi.org/project/boto3-stubs/).

Generated by [mypy-boto3-builder](https://github.com/vemel/mypy_boto3_builder).

## How to install

```bash
# Install type annotations for boto3 services you use
# mypy-boto3 should be installed as a dependency
python -m pip install 'boto3-stubs[s3,ec2]'
```

## Services type annotations
{% for service_name in package.service_names %}
- [{{ service_name.pypi_name }}](./{{ service_name.module_name }}) for [{{ service_name.class_name }}]({{ service_name.boto3_doc_link }}) service
{% endfor %}
