"""
Type annotations for {{ package.service_name.boto3_name }} service type definitions.

[Open documentation]({{ package.service_name.get_doc_link("type_defs") }})

Usage::

    ```python
    from {{ package.service_name.module_name }}.type_defs import {{ package.typed_dicts[0].name }}

    data: {{ package.typed_dicts[0].name }} = {...}
    ```
"""
{% for import_record in package.get_type_defs_required_import_records() -%}
    {% include "common/import_record_fallback.py.jinja2" with context %}
{% endfor -%}

{{ "\n\n" -}}

__all__ = (
{% for typed_dict in package.typed_dicts -%}
    {{ '"' -}}
    {{ typed_dict.name -}}
    {{ '"' -}}
    {{ ",\n" if not loop.last or loop.first else "\n" }}
{% endfor -%}
)

{% for typed_dict in package.typed_dicts -%}
    {% include "common/typed_dict.py.jinja2" with context -%}
    {{ "\n" -}}
{% endfor -%}
